﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition; // MEF - Ajouter la DLL dans les références !
using MEFInterface;
using System.IO; // interface des addins

namespace MEFExample
{
    class Program
    {
        [Import] // attribut marquant une propriété importée depuis les addins découverts
                 // la propriété est un énumérable car plusieurs addins peuvent alimenter la property
        public IEnumerable<ICompute> Operations { get; set; }

        // Entry point
        static void Main(string[] args)
        {
            new Program().Run();
        }

        // execute l'appli 
        private void Run()
        {
            Compose();
            Console.WriteLine("Operation Addins detected (test on x=5 and y=6) :");
            foreach (var operation in Operations)
            {
                Console.WriteLine(operation.OperationName+" : "+operation.Compute(5,6));
            }
            
            Console.ReadLine();
        }

        // gère les addins via MEF
        private void Compose()
        {
            // création du catalogue. Les addins sont ici sensés se trouver avec l'exe dans le même répertoire
            var catalog = new DirectoryPartCatalog(System.Reflection.Assembly.GetExecutingAssembly().Location);
            // créé le conteneur
            var container = new CompositionContainer(catalog.CreateResolver());
            // ajoute this pour qu'il puisse être lié via ses attributs 'import'
            container.AddPart(this); 
            // réalise la composition (connexion de tous les exports à tous les imports)
            container.Compose();
        }
    }
}
